@@ -3,6 +3,7 @@
 // Licensed under the MIT license, see the LICENSE file or <http://opensource.org/licenses/MIT>
 
 #![cfg_attr(not(feature = "std"), no_std)]
+#![allow(clippy::missing_safety_doc)]
 
 //! Safely cast bytes slices from/to slices of built-in fundamental numeric types.
 //!
@@ -61,33 +62,6 @@
 //!
 //! # }
 //! ```
-//! # Example with `Vec<T>`
-//! ```
-//! # extern crate byte_slice_cast;
-//! # fn main() {
-//! # #[cfg(feature = "alloc")] {
-//! use byte_slice_cast::*;
-//!
-//! let vec = vec![0x0102u16, 0x0304u16, 0x0506u16];
-//! let converted_vec = vec.into_byte_vec();
-//!
-//! if cfg!(target_endian = "big") {
-//!     assert_eq!(converted_vec, &[1, 2, 3, 4, 5, 6]);
-//! } else {
-//!     assert_eq!(converted_vec, &[2, 1, 4, 3, 6, 5]);
-//! }
-//!
-//! let converted_back_vec = converted_vec.into_vec_of::<u16>().unwrap();
-//! assert_eq!(&converted_back_vec[..], &[0x0102u16, 0x0304u16, 0x0506u16]);
-//! # }
-//! # }
-//! ```
-
-#[cfg(feature = "alloc")]
-#[macro_use]
-extern crate alloc;
-#[cfg(feature = "alloc")]
-use self::alloc::vec::*;
 
 use core::{fmt, mem, slice};
 
@@ -111,14 +85,6 @@ pub enum Error {
         src_slice_size: usize,
         dst_type_size: usize,
     },
-    /// When converting a `Vec<T>` it had a capacity that
-    /// allowed only for a non-integer number of values
-    /// from the output type.
-    CapacityMismatch {
-        dst_type: &'static str,
-        src_vec_capacity: usize,
-        dst_type_capacity: usize,
-    },
 }
 
 impl fmt::Display for Error {
@@ -150,20 +116,6 @@ impl fmt::Display for Error {
                     dst_type
                 )?;
             }
-            Error::CapacityMismatch {
-                dst_type,
-                src_vec_capacity,
-                dst_type_capacity,
-            } => {
-                write!(
-                    f,
-                    "cannot cast a vec into a vec::<{}>: the capacity ({}) of the vec is not divisible by the capacity of ({}) of {}",
-                    dst_type,
-                    src_vec_capacity,
-                    dst_type_capacity,
-                    dst_type
-                )?;
-            }
         }
 
         Ok(())
@@ -182,7 +134,6 @@ impl StdError for Error {
         match *self {
             AlignmentMismatch { .. } => "Alignment Mismatch",
             LengthMismatch { .. } => "Length Mismatch",
-            CapacityMismatch { .. } => "Capacity Mismatch",
         }
     }
 }
@@ -221,23 +172,6 @@ where
     Ok(size_out)
 }
 
-#[cfg(feature = "alloc")]
-fn check_capacity<U>(data: &Vec<u8>) -> Result<usize, Error>
-where
-    U: TypeName,
-{
-    let capacity_out = mem::size_of::<U>();
-    if data.capacity() % capacity_out != 0 {
-        let err = Error::CapacityMismatch {
-            dst_type: U::TYPE_NAME,
-            src_vec_capacity: data.capacity(),
-            dst_type_capacity: capacity_out,
-        };
-        return Err(err);
-    }
-    Ok(capacity_out)
-}
-
 fn check_constraints<U>(data: &[u8]) -> Result<usize, Error>
 where
     U: TypeName,
@@ -252,21 +186,137 @@ where
     Ok(data.len() / size_out)
 }
 
-#[cfg(feature = "alloc")]
-fn check_vec_constraints<U>(data: &Vec<u8>) -> Result<(usize, usize), Error>
-where
-    U: TypeName,
-{
-    if data.is_empty() {
-        return Ok((0, 0));
-    }
+macro_rules! impl_trait(
+    ($to:ty) => {
+        impl TypeName for $to {
+            const TYPE_NAME: &'static str = stringify!($to);
+        }
 
-    check_alignment::<[u8], U>(data)?;
-    let size_out = check_length::<[u8], U>(data)?;
-    let capacity_out = check_capacity::<U>(data)?;
+        unsafe impl FromByteSlice for $to {
+            #[inline]
+            fn from_byte_slice<T: AsRef<[u8]> + ?Sized>(slice: &T) -> Result<&[$to], Error> {
+                let slice = slice.as_ref();
+                let len = check_constraints::<$to>(slice)?;
 
-    Ok((data.len() / size_out, data.capacity() / capacity_out))
-}
+                // Need to handle the empty case separately as even an empty slices
+                // must have a correctly aligned data pointer
+                if len == 0 {
+                    Ok(&[])
+                } else {
+                    #[allow(clippy::cast_ptr_alignment)]
+                    unsafe {
+                        Ok(slice::from_raw_parts(slice.as_ptr() as *const $to, len))
+                    }
+                }
+            }
+
+            #[inline]
+            fn from_mut_byte_slice<T: AsMut<[u8]> + ?Sized>(slice: &mut T) -> Result<&mut [$to], Error> {
+                let slice = slice.as_mut();
+                let len = check_constraints::<$to>(slice)?;
+
+                // Need to handle the empty case separately as even an empty slices
+                // must have a correctly aligned data pointer
+                if len == 0 {
+                    Ok(&mut [])
+                } else {
+                    #[allow(clippy::cast_ptr_alignment)]
+                    unsafe {
+                        Ok(slice::from_raw_parts_mut(slice.as_mut_ptr() as *mut $to, len))
+                    }
+                }
+            }
+        }
+
+        unsafe impl ToByteSlice for $to {
+            #[inline]
+            fn to_byte_slice<T: AsRef<[$to]> + ?Sized>(slice: &T) -> &[u8] {
+                let slice = slice.as_ref();
+                let len = slice.len() * mem::size_of::<$to>();
+                unsafe {
+                    slice::from_raw_parts(slice.as_ptr() as *const u8, len)
+                }
+            }
+        }
+
+        unsafe impl ToMutByteSlice for $to {
+            #[inline]
+            fn to_mut_byte_slice<T: AsMut<[$to]> + ?Sized>(slice: &mut T) -> &mut [u8] {
+                let slice = slice.as_mut();
+                let len = slice.len() * mem::size_of::<$to>();
+                unsafe {
+                    slice::from_raw_parts_mut(slice.as_mut_ptr() as *mut u8, len)
+                }
+            }
+        }
+    };
+);
+
+macro_rules! impl_trait_array (
+    ($to:ty) => {
+        impl<const N: usize> TypeName for [$to; N] {
+            const TYPE_NAME: &'static str = stringify!([$to; N]);
+        }
+
+        unsafe impl<const N: usize> FromByteSlice for [$to; N] {
+            #[inline]
+            fn from_byte_slice<T: AsRef<[u8]> + ?Sized>(slice: &T) -> Result<&[[$to; N]], Error> {
+                let slice = slice.as_ref();
+                let len = check_constraints::<[$to; N]>(slice)?;
+
+                // Need to handle the empty case separately as even an empty slices
+                // must have a correctly aligned data pointer
+                if len == 0 {
+                    Ok(&[])
+                } else {
+                    #[allow(clippy::cast_ptr_alignment)]
+                    unsafe {
+                        Ok(slice::from_raw_parts(slice.as_ptr() as *const [$to; N], len))
+                    }
+                }
+            }
+
+            #[inline]
+            fn from_mut_byte_slice<T: AsMut<[u8]> + ?Sized>(slice: &mut T) -> Result<&mut [[$to; N]], Error> {
+                let slice = slice.as_mut();
+                let len = check_constraints::<[$to; N]>(slice)?;
+
+                // Need to handle the empty case separately as even an empty slices
+                // must have a correctly aligned data pointer
+                if len == 0 {
+                    Ok(&mut [])
+                } else {
+                    #[allow(clippy::cast_ptr_alignment)]
+                    unsafe {
+                        Ok(slice::from_raw_parts_mut(slice.as_mut_ptr() as *mut [$to; N], len))
+                    }
+                }
+            }
+        }
+
+        unsafe impl<const N: usize> ToByteSlice for [$to; N] {
+            #[inline]
+            fn to_byte_slice<T: AsRef<[[$to; N]]> + ?Sized>(slice: &T) -> &[u8] {
+                let slice = slice.as_ref();
+                let len = slice.len() * mem::size_of::<[$to; N]>();
+                unsafe {
+                    slice::from_raw_parts(slice.as_ptr() as *const u8, len)
+                }
+            }
+        }
+
+        unsafe impl<const N: usize> ToMutByteSlice for [$to; N] {
+            #[inline]
+            fn to_mut_byte_slice<T: AsMut<[[$to; N]]> + ?Sized>(slice: &mut T) -> &mut [u8] {
+                let slice = slice.as_mut();
+                let len = slice.len() * mem::size_of::<[$to; N]>();
+                unsafe {
+                    slice::from_raw_parts_mut(slice.as_mut_ptr() as *mut u8, len)
+                }
+            }
+        }
+    };
+);
 
 /// Trait for converting from a byte slice to a slice of a fundamental, built-in numeric type.
 ///
@@ -316,119 +366,6 @@ where
     fn to_mut_byte_slice<T: AsMut<[Self]> + ?Sized>(slice: &mut T) -> &mut [u8];
 }
 
-/// Trait for converting from a byte `Vec<u8>` to a `Vec<T>` of a fundamental, built-in numeric type.
-///
-/// This trait is an implementation detail. Use the [`IntoVecOf`] trait.
-///
-/// [`IntoVecOf`]: trait.IntoVecOf.html
-#[cfg(feature = "alloc")]
-pub unsafe trait FromByteVec
-where
-    Self: Sized,
-{
-    /// Convert from a byte `Vec<u8>` to a `Vec<T>` of a fundamental, built-in
-    /// numeric type
-    fn from_byte_vec(vec: Vec<u8>) -> Result<Vec<Self>, Error>;
-}
-
-macro_rules! impl_trait(
-    ($to:ty) => {
-        impl TypeName for $to {
-            const TYPE_NAME: &'static str = stringify!($to);
-        }
-
-        unsafe impl FromByteSlice for $to {
-            fn from_byte_slice<T: AsRef<[u8]> + ?Sized>(slice: &T) -> Result<&[$to], Error> {
-                let slice = slice.as_ref();
-                let len = check_constraints::<$to>(slice)?;
-
-                // Need to handle the empty case separately as even an empty slices
-                // must have a correctly aligned data pointer
-                if len == 0 {
-                    Ok(&[])
-                } else {
-                    #[allow(clippy::cast_ptr_alignment)]
-                    unsafe {
-                        Ok(slice::from_raw_parts(slice.as_ptr() as *const $to, len))
-                    }
-                }
-            }
-
-            fn from_mut_byte_slice<T: AsMut<[u8]> + ?Sized>(slice: &mut T) -> Result<&mut [$to], Error> {
-                let slice = slice.as_mut();
-                let len = check_constraints::<$to>(slice)?;
-
-                // Need to handle the empty case separately as even an empty slices
-                // must have a correctly aligned data pointer
-                if len == 0 {
-                    Ok(&mut [])
-                } else {
-                    #[allow(clippy::cast_ptr_alignment)]
-                    unsafe {
-                        Ok(slice::from_raw_parts_mut(slice.as_mut_ptr() as *mut $to, len))
-                    }
-                }
-            }
-        }
-
-        unsafe impl ToByteSlice for $to {
-            fn to_byte_slice<T: AsRef<[$to]> + ?Sized>(slice: &T) -> &[u8] {
-                let slice = slice.as_ref();
-                let len = slice.len() * mem::size_of::<$to>();
-                unsafe {
-                    slice::from_raw_parts(slice.as_ptr() as *const u8, len)
-                }
-            }
-        }
-
-        unsafe impl ToMutByteSlice for $to {
-            fn to_mut_byte_slice<T: AsMut<[$to]> + ?Sized>(slice: &mut T) -> &mut [u8] {
-                let slice = slice.as_mut();
-                let len = slice.len() * mem::size_of::<$to>();
-                unsafe {
-                    slice::from_raw_parts_mut(slice.as_mut_ptr() as *mut u8, len)
-                }
-            }
-        }
-
-        #[cfg(feature = "alloc")]
-        unsafe impl FromByteVec for $to {
-            fn from_byte_vec(mut vec: Vec<u8>) -> Result<Vec<$to>, Error> {
-                let (len, capacity) = check_vec_constraints::<$to>(&vec)?;
-                if capacity == 0 {
-                    Ok(vec![])
-                }
-                else {
-                    let ptr = vec.as_mut_ptr();
-                    mem::forget(vec);
-                    unsafe {
-                        Ok(Vec::from_raw_parts(ptr as *mut $to, len, capacity))
-                    }
-                }
-            }
-        }
-
-        #[cfg(feature = "alloc")]
-        impl IntoByteVec for Vec<$to> {
-            fn into_byte_vec(mut self) -> Vec<u8> {
-                if self.capacity() == 0 {
-                    vec![]
-                }
-                else {
-                    let size = mem::size_of::<$to>();
-                    let len = self.len() * size;
-                    let capacity = self.capacity() * size;
-                    let ptr = self.as_mut_ptr();
-                    mem::forget(self);
-                    unsafe {
-                        Vec::from_raw_parts(ptr as *mut u8, len, capacity)
-                    }
-                }
-            }
-        }
-    };
-);
-
 /// Trait for converting from a byte slice to a slice of a fundamental, built-in numeric type.
 ///
 /// # Example
@@ -452,6 +389,7 @@ pub trait AsSliceOf {
 }
 
 impl<U: AsRef<[u8]> + ?Sized> AsSliceOf for U {
+    #[inline]
     fn as_slice_of<T: FromByteSlice>(&self) -> Result<&[T], Error> {
         FromByteSlice::from_byte_slice(self)
     }
@@ -481,6 +419,7 @@ pub trait AsMutSliceOf {
 }
 
 impl<U: AsMut<[u8]> + ?Sized> AsMutSliceOf for U {
+    #[inline]
     fn as_mut_slice_of<T: FromByteSlice>(&mut self) -> Result<&mut [T], Error> {
         FromByteSlice::from_mut_byte_slice(self)
     }
@@ -510,6 +449,7 @@ pub trait AsByteSlice<T> {
 }
 
 impl<T: ToByteSlice, U: AsRef<[T]> + ?Sized> AsByteSlice<T> for U {
+    #[inline]
     fn as_byte_slice(&self) -> &[u8] {
         ToByteSlice::to_byte_slice(self)
     }
@@ -539,66 +479,12 @@ pub trait AsMutByteSlice<T> {
 }
 
 impl<T: ToMutByteSlice, U: AsMut<[T]> + ?Sized> AsMutByteSlice<T> for U {
+    #[inline]
     fn as_mut_byte_slice(&mut self) -> &mut [u8] {
         ToMutByteSlice::to_mut_byte_slice(self)
     }
 }
 
-/// Trait for converting from a byte `Vec<u8>` to a `Vec<T>` of a fundamental, built-in
-/// numeric type.
-///
-/// # Example
-/// ```no_run
-/// # extern crate byte_slice_cast;
-/// # fn main() {
-/// use byte_slice_cast::*;
-///
-/// let mut vec: Vec<u8> = vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8];
-/// let converted_vec = vec.into_vec_of::<u16>().unwrap();
-///
-/// if cfg!(target_endian = "big") {
-///     assert_eq!(converted_vec, vec![0x0102, 0x0304, 0x0506]);
-/// } else {
-///     assert_eq!(converted_vec, vec![0x0201, 0x0403, 0x0605]);
-/// }
-/// # }
-/// ```
-#[cfg(feature = "alloc")]
-pub trait IntoVecOf {
-    fn into_vec_of<T: FromByteVec>(self) -> Result<Vec<T>, Error>;
-}
-
-#[cfg(feature = "alloc")]
-impl IntoVecOf for Vec<u8> {
-    fn into_vec_of<T: FromByteVec>(self) -> Result<Vec<T>, Error> {
-        FromByteVec::from_byte_vec(self)
-    }
-}
-
-/// Trait for converting from a `Vec<T>` of a fundamental, built-in numeric type to a
-/// byte `Vec<u8>`.
-///
-/// # Example
-/// ```no_run
-/// # extern crate byte_slice_cast;
-/// # fn main() {
-/// use byte_slice_cast::*;
-///
-/// let mut vec: Vec<u16> = vec![0x0102, 0x0304, 0x0506];
-/// let converted_vec = vec.into_byte_vec();
-///
-/// if cfg!(target_endian = "big") {
-///     assert_eq!(converted_vec, vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8]);
-/// } else {
-///     assert_eq!(converted_vec, vec![2u8, 1u8, 4u8, 3u8, 6u8, 5u8]);
-/// }
-/// # }
-/// ```
-#[cfg(feature = "alloc")]
-pub trait IntoByteVec {
-    fn into_byte_vec(self) -> Vec<u8>;
-}
-
 impl_trait!(u8);
 impl_trait!(u16);
 impl_trait!(u32);
@@ -614,6 +500,39 @@ impl_trait!(f64);
 impl_trait!(usize);
 impl_trait!(isize);
 
+impl_trait_array!(u8);
+impl_trait_array!(u16);
+impl_trait_array!(u32);
+impl_trait_array!(u64);
+impl_trait_array!(u128);
+impl_trait_array!(i8);
+impl_trait_array!(i16);
+impl_trait_array!(i32);
+impl_trait_array!(i64);
+impl_trait_array!(i128);
+impl_trait_array!(f32);
+impl_trait_array!(f64);
+impl_trait_array!(usize);
+impl_trait_array!(isize);
+
+impl TypeName for () {
+    const TYPE_NAME: &'static str = "()";
+}
+
+unsafe impl ToByteSlice for () {
+    #[inline]
+    fn to_byte_slice<T: AsRef<[()]> + ?Sized>(_: &T) -> &[u8] {
+        &[]
+    }
+}
+
+unsafe impl ToMutByteSlice for () {
+    #[inline]
+    fn to_mut_byte_slice<T: AsMut<[()]> + ?Sized>(_: &mut T) -> &mut [u8] {
+        &mut []
+    }
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -629,17 +548,6 @@ mod tests {
         assert_eq!(&input, output2);
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn u8_vec() {
-        let input: Vec<u8> = vec![0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15];
-        let output = input.clone().into_vec_of::<u8>().unwrap();
-        assert_eq!(input, output);
-
-        let output2: Vec<u8> = input.clone().into_byte_vec();
-        assert_eq!(input, output2);
-    }
-
     #[test]
     fn u16() {
         let slice: [u16; 8] = [0, 1, 2, 3, 4, 5, 6, 7];
@@ -669,44 +577,6 @@ mod tests {
         assert_eq!(bytes.as_slice_of::<u16>(), Ok(slice.as_ref()));
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn u16_into_vec() {
-        let slice: [u16; 8] = [0, 1, 2, 3, 4, 5, 6, 7];
-        let vec: Vec<u16> = Vec::from(slice.as_ref());
-        let byte_vec = vec.clone().into_byte_vec();
-
-        if cfg!(target_endian = "big") {
-            assert_eq!(&byte_vec, &[0, 0, 0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7]);
-        } else {
-            assert_eq!(&byte_vec, &[0, 0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0, 6, 0, 7, 0]);
-        }
-
-        let err_vec = vec![1u8, 2u8, 3u8];
-        assert_eq!(
-            err_vec.clone().into_vec_of::<u16>(),
-            Err(Error::LengthMismatch {
-                dst_type: "u16",
-                src_slice_size: 3,
-                dst_type_size: mem::size_of::<u16>(),
-            })
-        );
-
-        let mut err_vec: Vec<u8> = Vec::with_capacity(13);
-        err_vec.append(&mut vec![1u8, 2u8]);
-        assert_eq!(err_vec.capacity(), 13);
-        assert_eq!(
-            err_vec.into_vec_of::<u16>(),
-            Err(Error::CapacityMismatch {
-                dst_type: "u16",
-                src_vec_capacity: 13,
-                dst_type_capacity: mem::size_of::<u16>(),
-            })
-        );
-
-        assert_eq!(byte_vec.into_vec_of::<u16>(), Ok(vec));
-    }
-
     #[cfg(feature = "std")]
     #[test]
     fn u16_error_string() {
@@ -757,43 +627,6 @@ mod tests {
         assert_eq!(bytes.as_slice_of::<u32>(), Ok(slice.as_ref()));
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn u32_vec() {
-        let slice: [u32; 4] = [0, 1, 2, 3];
-        let vec: Vec<u32> = Vec::from(slice.as_ref());
-        let byte_vec = vec.clone().into_byte_vec();
-
-        if cfg!(target_endian = "big") {
-            assert_eq!(&byte_vec, &[0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3]);
-        } else {
-            assert_eq!(&byte_vec, &[0, 0, 0, 0, 1, 0, 0, 0, 2, 0, 0, 0, 3, 0, 0, 0]);
-        }
-
-        let err_vec = vec![1u8, 2u8, 3u8, 4u8, 5u8];
-        assert_eq!(
-            err_vec.clone().into_vec_of::<u32>(),
-            Err(Error::LengthMismatch {
-                dst_type: "u32",
-                src_slice_size: 5,
-                dst_type_size: mem::size_of::<u32>(),
-            })
-        );
-
-        let mut err_vec: Vec<u8> = Vec::with_capacity(13);
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8]);
-        assert_eq!(err_vec.capacity(), 13);
-        assert_eq!(
-            err_vec.into_vec_of::<u32>(),
-            Err(Error::CapacityMismatch {
-                dst_type: "u32",
-                src_vec_capacity: 13,
-                dst_type_capacity: mem::size_of::<u32>(),
-            })
-        );
-        assert_eq!(byte_vec.into_vec_of::<u32>(), Ok(vec));
-    }
-
     #[test]
     fn u64() {
         let slice: [u64; 2] = [0, 1];
@@ -823,44 +656,9 @@ mod tests {
         assert_eq!(bytes.as_slice_of::<u64>(), Ok(slice.as_ref()));
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn u64_vec() {
-        let slice: [u64; 2] = [0, 1];
-        let vec: Vec<u64> = Vec::from(slice.as_ref());
-        let byte_vec = vec.clone().into_byte_vec();
-
-        if cfg!(target_endian = "big") {
-            assert_eq!(&byte_vec, &[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]);
-        } else {
-            assert_eq!(&byte_vec, &[0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]);
-        }
-
-        let err_vec = vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8, 9u8];
-        assert_eq!(
-            err_vec.clone().into_vec_of::<u64>(),
-            Err(Error::LengthMismatch {
-                dst_type: "u64",
-                src_slice_size: 9,
-                dst_type_size: mem::size_of::<f64>()
-            })
-        );
-
-        let mut err_vec: Vec<u8> = Vec::with_capacity(13);
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8]);
-        assert_eq!(err_vec.capacity(), 13);
-        assert_eq!(
-            err_vec.into_vec_of::<u64>(),
-            Err(Error::CapacityMismatch {
-                dst_type: "u64",
-                src_vec_capacity: 13,
-                dst_type_capacity: mem::size_of::<u64>()
-            })
-        );
-        assert_eq!(byte_vec.into_vec_of::<u64>(), Ok(vec));
-    }
-
     #[test]
+    #[allow(clippy::collapsible_if)]
+    #[allow(clippy::collapsible_else_if)]
     fn usize() {
         let slice: [usize; 2] = [0, 1];
         let bytes = slice.as_byte_slice();
@@ -905,61 +703,6 @@ mod tests {
         assert_eq!(bytes.as_slice_of::<usize>(), Ok(slice.as_ref()));
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn usize_vec() {
-        let slice: [usize; 2] = [0, 1];
-        let vec: Vec<usize> = Vec::from(slice.as_ref());
-        let byte_vec = vec.clone().into_byte_vec();
-
-        if cfg!(target_endian = "big") {
-            if cfg!(target_pointer_width = "16") {
-                assert_eq!(&byte_vec, &[0, 0, 0, 1]);
-            } else if cfg!(target_pointer_width = "32") {
-                assert_eq!(&byte_vec, &[0, 0, 0, 0, 0, 0, 0, 1]);
-            } else if cfg!(target_pointer_width = "64") {
-                assert_eq!(&byte_vec, &[0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1]);
-            } else {
-                panic!("Unhandled target_endian/target_pointer_width configuration");
-            }
-        } else {
-            if cfg!(target_pointer_width = "16") {
-                assert_eq!(&byte_vec, &[0, 0, 1, 0]);
-            } else if cfg!(target_pointer_width = "32") {
-                assert_eq!(&byte_vec, &[0, 0, 0, 0, 1, 0, 0, 0]);
-            } else if cfg!(target_pointer_width = "64") {
-                assert_eq!(&byte_vec, &[0, 0, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 0, 0]);
-            } else {
-                panic!("Unhandled target_endian/target_pointer_width configuration");
-            }
-        }
-
-        let mut err_vec = vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8];
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8, 9u8]);
-        assert_eq!(
-            err_vec.clone().into_vec_of::<usize>(),
-            Err(Error::LengthMismatch {
-                dst_type: "usize",
-                src_slice_size: 17,
-                dst_type_size: mem::size_of::<usize>()
-            })
-        );
-
-        let mut err_vec: Vec<u8> = Vec::with_capacity(21);
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8]);
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8]);
-        assert_eq!(err_vec.capacity(), 21);
-        assert_eq!(
-            err_vec.into_vec_of::<usize>(),
-            Err(Error::CapacityMismatch {
-                dst_type: "usize",
-                src_vec_capacity: 21,
-                dst_type_capacity: mem::size_of::<usize>()
-            })
-        );
-        assert_eq!(byte_vec.into_vec_of::<usize>(), Ok(vec));
-    }
-
     #[test]
     fn f32() {
         let slice: [f32; 4] = [2.0, 1.0, 0.5, 0.25];
@@ -1001,55 +744,6 @@ mod tests {
         assert_eq!(bytes.as_slice_of::<f32>(), Ok(slice.as_ref()));
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn f32_vec() {
-        let slice: [f32; 4] = [2.0, 1.0, 0.5, 0.25];
-        let vec: Vec<f32> = Vec::from(slice.as_ref());
-        let byte_vec = vec.clone().into_byte_vec();
-
-        if cfg!(target_endian = "big") {
-            assert_eq!(
-                byte_vec,
-                [
-                    0x40, 0x00, 0x00, 0x00, 0x3f, 0x80, 0x00, 0x00, 0x3f, 0x00, 0x00, 0x00, 0x3e,
-                    0x80, 0x00, 0x00
-                ]
-            );
-        } else {
-            assert_eq!(
-                byte_vec,
-                [
-                    0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x80, 0x3f, 0x00, 0x00, 0x00, 0x3f, 0x00,
-                    0x00, 0x80, 0x3e
-                ]
-            );
-        };
-
-        let err_vec = vec![1u8, 2u8, 3u8, 4u8, 5u8];
-        assert_eq!(
-            err_vec.clone().into_vec_of::<f32>(),
-            Err(Error::LengthMismatch {
-                dst_type: "f32",
-                src_slice_size: 5,
-                dst_type_size: mem::size_of::<f32>(),
-            })
-        );
-
-        let mut err_vec: Vec<u8> = Vec::with_capacity(13);
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8]);
-        assert_eq!(err_vec.capacity(), 13);
-        assert_eq!(
-            err_vec.into_vec_of::<f32>(),
-            Err(Error::CapacityMismatch {
-                dst_type: "f32",
-                src_vec_capacity: 13,
-                dst_type_capacity: mem::size_of::<f32>(),
-            })
-        );
-        assert_eq!(byte_vec.into_vec_of::<f32>(), Ok(vec));
-    }
-
     #[test]
     fn f64() {
         let slice: [f64; 2] = [2.0, 0.5];
@@ -1091,55 +785,6 @@ mod tests {
         assert_eq!(bytes.as_slice_of::<f64>(), Ok(slice.as_ref()));
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn f64_vec() {
-        let slice: [f64; 2] = [2.0, 0.5];
-        let vec: Vec<f64> = Vec::from(slice.as_ref());
-        let byte_vec = vec.clone().into_byte_vec();
-
-        if cfg!(target_endian = "big") {
-            assert_eq!(
-                byte_vec,
-                [
-                    0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x3f, 0xe0, 0x00, 0x00, 0x00,
-                    0x00, 0x00, 0x00
-                ]
-            );
-        } else {
-            assert_eq!(
-                byte_vec,
-                [
-                    0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00,
-                    0x00, 0xe0, 0x3f
-                ]
-            );
-        };
-
-        let err_vec = vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8, 9u8];
-        assert_eq!(
-            err_vec.clone().into_vec_of::<f64>(),
-            Err(Error::LengthMismatch {
-                dst_type: "f64",
-                src_slice_size: 9,
-                dst_type_size: 8
-            })
-        );
-
-        let mut err_vec: Vec<u8> = Vec::with_capacity(13);
-        err_vec.append(&mut vec![1u8, 2u8, 3u8, 4u8, 5u8, 6u8, 7u8, 8u8]);
-        assert_eq!(err_vec.capacity(), 13);
-        assert_eq!(
-            err_vec.into_vec_of::<f64>(),
-            Err(Error::CapacityMismatch {
-                dst_type: "f64",
-                src_vec_capacity: 13,
-                dst_type_capacity: mem::size_of::<f64>()
-            })
-        );
-        assert_eq!(byte_vec.into_vec_of::<f64>(), Ok(vec));
-    }
-
     #[test]
     fn u16_mut() {
         let mut slice: [u16; 8] = [0, 1, 2, 3, 4, 5, 6, 7];
@@ -1300,14 +945,6 @@ mod tests {
         assert_eq!(bytes, &[]);
     }
 
-    #[cfg(feature = "alloc")]
-    #[test]
-    fn u16_empty_vec_into_byte_vec() {
-        let vec: Vec<u16> = vec![];
-        let byte_vec = vec.clone().into_byte_vec();
-        assert_eq!(byte_vec, vec![]);
-    }
-
     #[test]
     fn u16_empty_from_byte_slice() {
         let bytes: [u8; 0] = [];
@@ -1315,11 +952,54 @@ mod tests {
         assert_eq!(slice, &[]);
     }
 
-    #[cfg(feature = "alloc")]
     #[test]
-    fn byte_vec_into_u16_empty_vec() {
-        let byte_vec: Vec<u8> = vec![];
-        let vec = byte_vec.clone().into_vec_of::<u16>().unwrap();
-        assert_eq!(vec, vec![]);
+    fn unit() {
+        let slice: [(); 4] = [(), (), (), ()];
+        let bytes = slice.as_byte_slice();
+
+        assert_eq!(bytes, &[]);
+    }
+
+    #[test]
+    fn u8_array() {
+        let input: &[[u8; 3]] = &[[0, 1, 2], [3, 4, 5], [6, 7, 8], [9, 10, 11], [12, 13, 14]][..];
+
+        let bytes: &[u8] = input.as_byte_slice();
+        assert_eq!(bytes, [0, 1, 2, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14]);
+
+        let output = bytes.as_slice_of::<[u8; 3]>().unwrap();
+
+        assert_eq!(output, input);
+    }
+
+    #[test]
+    fn u16_array() {
+        let input: &[[u16; 3]] = &[[0, 1, 2], [3, 4, 5]][..];
+
+        let bytes: &[u8] = input.as_byte_slice();
+        if cfg!(target_endian = "big") {
+            assert_eq!(bytes, [0, 0, 0, 1, 0, 2, 0, 3, 0, 4, 0, 5]);
+        } else {
+            assert_eq!(bytes, [0, 0, 1, 0, 2, 0, 3, 0, 4, 0, 5, 0]);
+        };
+
+        assert_eq!(
+            (&bytes[1..]).as_slice_of::<[u16; 3]>(),
+            Err(Error::AlignmentMismatch {
+                dst_type: "[u16 ; N]",
+                dst_minimum_alignment: mem::align_of::<[u16; 3]>()
+            })
+        );
+        assert_eq!(
+            (&bytes[0..4]).as_slice_of::<[u16; 3]>(),
+            Err(Error::LengthMismatch {
+                dst_type: "[u16 ; N]",
+                src_slice_size: 4,
+                dst_type_size: 6
+            })
+        );
+
+        let output = bytes.as_slice_of::<[u16; 3]>().unwrap();
+        assert_eq!(output, input);
     }
 }
